@@ -26,14 +26,16 @@
 #include <stdio.h>
 #include "as.h"
 #include "bfd.h"
+#define DEFINE_TABLE
+#define h8_opcodes ops
 #include "opcode/h8300.h"
 #include <ctype.h>
 
 const char comment_chars[] =
 {';', 0};
 const char line_separator_chars[] =
-{'$', 0};
-const char line_comment_chars[] = "";
+{0};
+const char line_comment_chars[] = "#";
 
 /* This table describes all the machine specific pseudo-ops the assembler
    has to support.  The fields are:
@@ -44,8 +46,34 @@ const char line_comment_chars[] = "";
 
 void cons ();
 
+int Hmode;
+#define PSIZE (Hmode ? L_32 : L_16)
+#define DMODE (L_16)
+#define DSYMMODE (Hmode ? L_24 : L_16)
+int bsize = L_8;		/* default branch displacement */
+
+
+void
+h8300hmode ()
+{
+  Hmode = 1;
+}
+
+
+void
+sbranch (size)
+     int size;
+{
+  bsize = size;
+}
+
 const pseudo_typeS md_pseudo_table[] =
 {
+
+  {"h8300h", h8300hmode, 0},
+  {"sbranch", sbranch, L_8},
+  {"lbranch", sbranch, L_16},
+
   {"int", cons, 2},
   {"data.b", cons, 1},
   {"data.w", cons, 2},
@@ -58,7 +86,7 @@ const pseudo_typeS md_pseudo_table[] =
   {0, 0, 0}
 };
 
-int md_reloc_size;
+const int md_reloc_size;
 
 const char EXP_CHARS[] = "eE";
 
@@ -77,57 +105,6 @@ static struct hash_control *opcode_hash_control;	/* Opcode mnemonics */
   This function is called once, at assembler startup time.  This should
   set up all the tables, etc that the MD part of the assembler needs
   */
-#if 0
-/* encode the size and number into the number field
-   xxnnnn
-   00    8 bit
-   01    16 bit
-   10    ccr
-   nnnnreg number
-   */
-#define WORD_REG 0x10
-#define BYTE_REG 0x00
-#define CCR_REG  0x20
-struct reg_entry
-{
-  char *name;
-  char number;
-};
-
-struct reg_entry reg_list[] =
-{
-  "r0", WORD_REG + 0,
-  "r1", WORD_REG + 1,
-  "r2", WORD_REG + 2,
-  "r3", WORD_REG + 3,
-  "r4", WORD_REG + 4,
-  "r5", WORD_REG + 5,
-  "r6", WORD_REG + 6,
-  "r7", WORD_REG + 7,
-  "fp", WORD_REG + 6,
-  "sp", WORD_REG + 7,
-  "r0h", BYTE_REG + 0,
-  "r0l", BYTE_REG + 1,
-  "r1h", BYTE_REG + 2,
-  "r1l", BYTE_REG + 3,
-  "r2h", BYTE_REG + 4,
-  "r2l", BYTE_REG + 5,
-  "r3h", BYTE_REG + 6,
-  "r3l", BYTE_REG + 7,
-  "r4h", BYTE_REG + 8,
-  "r4l", BYTE_REG + 9,
-  "r5h", BYTE_REG + 10,
-  "r5l", BYTE_REG + 11,
-  "r6h", BYTE_REG + 12,
-  "r6l", BYTE_REG + 13,
-  "r7h", BYTE_REG + 14,
-  "r7l", BYTE_REG + 15,
-  "ccr", CCR_REG,
-  0, 0
-}
- ;
-
-#endif
 
 
 void
@@ -156,13 +133,13 @@ md_begin ()
 	{
 	  if (*src == '.')
 	    {
-	      *dst++ = 0;
 	      src++;
 	      opcode->size = *src;
 	      break;
 	    }
 	  *dst++ = *src++;
 	}
+      *dst++ = 0;
       if (strcmp (buffer, prev_buffer))
 	{
 	  hash_insert (opcode_hash_control, buffer, (char *) opcode);
@@ -191,9 +168,12 @@ struct h8_exp
   char *e_end;
   expressionS e_exp;
 };
+int dispreg;
+int opsize;			/* Set when a register size is seen */
+
+
 struct h8_op
 {
-  unsigned int dispreg;
   op_type mode;
   unsigned reg;
   expressionS exp;
@@ -210,28 +190,18 @@ struct h8_op
 
   */
 
-op_type r8_sord[] =
-{RS8, RD8};
-op_type r16_sord[] =
-{RS16, RD16};
-op_type rind_sord[] =
-{RSIND, RDIND};
-op_type abs_sord[2] =
-{ABS16SRC, ABS16DST};
-op_type disp_sord[] =
-{DISPSRC, DISPDST};
-
 /* try and parse a reg name, returns number of chars consumed */
 int
-DEFUN (parse_reg, (src, mode, reg, dst),
-       char *src AND
-       op_type * mode AND
-       unsigned int *reg AND
-       int dst)
+parse_reg (src, mode, reg, direction)
+     char *src;
+     op_type *mode;
+     unsigned int *reg;
+     int direction;
+
 {
   if (src[0] == 's' && src[1] == 'p')
     {
-      *mode = r16_sord[dst];
+      *mode = PSIZE | REG | direction;
       *reg = 7;
       return 2;
     }
@@ -243,27 +213,48 @@ DEFUN (parse_reg, (src, mode, reg, dst),
     }
   if (src[0] == 'f' && src[1] == 'p')
     {
-      *mode = r16_sord[dst];
+      *mode = PSIZE | REG | direction;
       *reg = 6;
       return 2;
     }
+  if (src[0] == 'e'
+      && src[1] == 'r'
+      && src[2] >= '0' && src[2] <= '7')
+    {
+      *mode = L_32 | REG | direction;
+      *reg = src[2] - '0';
+      if (!Hmode)
+	as_warn ("Reg only legal for H8/300-H");
+
+      return 3;
+    }
+  if (src[0] == 'e'
+      && src[1] >= '0' && src[1] <= '7')
+    {
+      *mode = L_16 | REG | direction;
+      *reg = src[1] - '0' + 8;
+      if (!Hmode)
+	as_warn ("Reg only legal for H8/300-H");
+      return 2;
+    }
+
   if (src[0] == 'r')
     {
       if (src[1] >= '0' && src[1] <= '7')
 	{
 	  if (src[2] == 'l')
 	    {
-	      *mode = r8_sord[dst];
+	      *mode = L_8 | REG | direction;
 	      *reg = (src[1] - '0') + 8;
 	      return 3;
 	    }
 	  if (src[2] == 'h')
 	    {
-	      *mode = r8_sord[dst];
+	      *mode = L_8 | REG | direction;
 	      *reg = (src[1] - '0');
 	      return 3;
 	    }
-	  *mode = r16_sord[dst];
+	  *mode = L_16 | REG | direction;
 	  *reg = (src[1] - '0');
 	  return 2;
 	}
@@ -278,35 +269,21 @@ DEFUN (parse_exp, (s, op),
 {
   char *save = input_line_pointer;
   char *new;
-  segT seg;
 
   input_line_pointer = s;
-  seg = expr (0, op);
+  expression (op);
+  if (op->X_op == O_absent)
+    as_bad ("missing operand");
   new = input_line_pointer;
   input_line_pointer = save;
-  if (SEG_NORMAL (seg))
-    return new;
-  switch (seg)
-    {
-    case SEG_ABSOLUTE:
-    case SEG_UNKNOWN:
-    case SEG_DIFFERENCE:
-    case SEG_BIG:
-    case SEG_REGISTER:
-      return new;
-    case SEG_ABSENT:
-      as_bad ("Missing operand");
-      return new;
-    default:
-      as_bad ("Don't understand operand of type %s", segment_name (seg));
-      return new;
-    }
+  return new;
 }
 
 static char *
-DEFUN (skip_colonthing, (ptr, exp),
-       char *ptr AND
-       expressionS * exp)
+skip_colonthing (ptr, exp, mode)
+     char *ptr;
+     expressionS *exp;
+     int *mode;
 {
   if (*ptr == ':')
     {
@@ -319,6 +296,15 @@ DEFUN (skip_colonthing, (ptr, exp),
 	}
       else
 	{
+	  *mode &= ~SIZE;
+	  if (*ptr == '2')
+	    {
+	      *mode |= L_24;
+	    }
+	  else if (*ptr == '1')
+	    {
+	      *mode |= L_16;
+	    }
 	  while (isdigit (*ptr))
 	    ptr++;
 	}
@@ -343,11 +329,44 @@ DEFUN (skip_colonthing, (ptr, exp),
 
    */
 
+char *
+colonmod24 (op, src)
+     struct h8_op *op;
+     char *src;
+
+{
+  int mode = 0;
+  src = skip_colonthing (src, &op->exp, &mode);
+
+  if (!mode)
+    {
+      /* Choose a default mode */
+      if (op->exp.X_add_number < -32768
+	  || op->exp.X_add_number > 32767)
+	{
+	  if (Hmode)
+	    mode = L_24;
+	  else
+	    mode = L_16;
+	}
+      else if (op->exp.X_add_symbol
+	       || op->exp.X_op_symbol)
+	mode = DSYMMODE;
+      else
+	mode = DMODE;
+    }
+  op->mode |= mode;
+  return src;
+
+}
+
+
 static void
-DEFUN (get_operand, (ptr, op, dst),
-       char **ptr AND
-       struct h8_op *op AND
-       unsigned int dst)
+get_operand (ptr, op, dst, direction)
+     char **ptr;
+     struct h8_op *op;
+     unsigned int dst;
+
 {
   char *src = *ptr;
   op_type mode;
@@ -357,7 +376,7 @@ DEFUN (get_operand, (ptr, op, dst),
 
   op->mode = E;
 
-  len = parse_reg (src, &op->mode, &op->reg, dst);
+  len = parse_reg (src, &op->mode, &op->reg, direction);
   if (len)
     {
       *ptr = src + len;
@@ -371,7 +390,8 @@ DEFUN (get_operand, (ptr, op, dst),
 	{
 	  src++;
 	  src = parse_exp (src, &op->exp);
-	  src = skip_colonthing (src, &op->exp);
+
+	  src = skip_colonthing (src, &op->exp, &op->mode);
 
 	  *ptr = src;
 
@@ -384,43 +404,48 @@ DEFUN (get_operand, (ptr, op, dst),
       if (*src == '-')
 	{
 	  src++;
-	  len = parse_reg (src, &mode, &num, dst);
+	  len = parse_reg (src, &mode, &num, direction);
 	  if (len == 0)
 	    {
 	      /* Oops, not a reg after all, must be ordinary exp */
 	      src--;
 	      /* must be a symbol */
-	      op->mode = abs_sord[dst];
-	      *ptr = skip_colonthing (parse_exp (src, &op->exp), &op->exp);
+	      op->mode = ABS | PSIZE | direction;
+	      *ptr = skip_colonthing (parse_exp (src, &op->exp),
+				      &op->exp, &op->mode);
 
 	      return;
 
 
 	    }
 
-	  if (mode != r16_sord[dst])
-	    {
-	      as_bad ("@- needs word register");
-	    }
+
+	  if ((mode & SIZE) != PSIZE)
+	    as_bad ("Wrong size pointer register for architecture.");
 	  op->mode = RDDEC;
 	  op->reg = num;
 	  *ptr = src + len;
 	  return;
 	}
-      if (*src == '(' && ')')
+      if (*src == '(')
 	{
 	  /* Disp */
 	  src++;
+
+	  /* Start off assuming a 16 bit offset */
+
+
 	  src = parse_exp (src, &op->exp);
 
+	  src = colonmod24 (op, src);
+
 	  if (*src == ')')
 	    {
 	      src++;
-	      op->mode = abs_sord[dst];
+	      op->mode |= ABS | direction;
 	      *ptr = src;
 	      return;
 	    }
-	  src = skip_colonthing (src, &op->exp);
 
 	  if (*src != ',')
 	    {
@@ -429,15 +454,17 @@ DEFUN (get_operand, (ptr, op, dst),
 
 	    }
 	  src++;
-	  len = parse_reg (src, &mode, &op->reg, dst);
-	  if (len == 0 || mode != r16_sord[dst])
+
+	  len = parse_reg (src, &mode, &op->reg, direction);
+	  if (len == 0 || !(mode & REG))
 	    {
 	      as_bad ("expected @(exp, reg16)");
 	      return;
 	    }
-	  op->mode = disp_sord[dst];
+	  op->mode |= DISP | direction;
+	  dispreg = op->reg;
 	  src += len;
-	  src = skip_colonthing (src, &op->exp);
+	  src = skip_colonthing (src, &op->exp, &op->mode);
 
 	  if (*src != ')' && '(')
 	    {
@@ -448,7 +475,7 @@ DEFUN (get_operand, (ptr, op, dst),
 
 	  return;
 	}
-      len = parse_reg (src, &mode, &num, dst);
+      len = parse_reg (src, &mode, &num, direction);
 
       if (len)
 	{
@@ -456,23 +483,17 @@ DEFUN (get_operand, (ptr, op, dst),
 	  if (*src == '+')
 	    {
 	      src++;
-	      if (mode != RS16)
-		{
-		  as_bad ("@Rn+ needs src word register");
-		  return;
-		}
+	      if ((mode & SIZE) != PSIZE)
+		as_bad ("Wrong size pointer register for architecture.");
 	      op->mode = RSINC;
 	      op->reg = num;
 	      *ptr = src;
 	      return;
 	    }
-	  if (mode != r16_sord[dst])
-	    {
-	      as_bad ("@Rn needs word register");
-	      return;
+	  if ((mode & SIZE) != PSIZE)
+	    as_bad ("Wrong size pointer register for architecture.");
 
-	    }
-	  op->mode = rind_sord[dst];
+	  op->mode = direction | IND | PSIZE;
 	  op->reg = num;
 	  *ptr = src;
 
@@ -481,8 +502,11 @@ DEFUN (get_operand, (ptr, op, dst),
       else
 	{
 	  /* must be a symbol */
-	  op->mode = abs_sord[dst];
-	  *ptr = skip_colonthing (parse_exp (src, &op->exp), &op->exp);
+
+	  op->mode = ABS | direction;
+	  src = parse_exp (src, &op->exp);
+
+	  *ptr = colonmod24 (op, src);
 
 	  return;
 	}
@@ -492,16 +516,38 @@ DEFUN (get_operand, (ptr, op, dst),
   if (*src == '#')
     {
       src++;
-      op->mode = IMM16;
+      op->mode = IMM;
       src = parse_exp (src, &op->exp);
-      *ptr = skip_colonthing (src, &op->exp);
+      *ptr = skip_colonthing (src, &op->exp, &op->mode);
 
       return;
     }
   else
     {
-      *ptr = parse_exp (src, &op->exp);
-      op->mode = DISP8;
+      src = parse_exp (src, &op->exp);
+      /* Trailing ':' size ? */
+      if (*src == ':')
+	{
+	  if (src[1] == '1' && src[2] == '6')
+	    {
+	      op->mode = PCREL | L_16;
+	      src += 3;
+	    }
+	  else if (src[1] == '8')
+	    {
+	      op->mode = PCREL | L_8;
+	      src += 2;
+	    }
+	  else
+	    {
+	      as_bad ("expect :8 or :16 here");
+	    }
+	}
+      else
+	{
+	  op->mode = PCREL | bsize;
+	}
+      *ptr = src;
     }
 }
 
@@ -524,16 +570,24 @@ DEFUN (get_operands, (noperands, op_end, operand),
 
     case 1:
       ptr++;
-      get_operand (&ptr, operand + 0, 0);
-      operand[1].mode = 0;
-      break;
+      get_operand (&ptr, operand + 0, 0, SRC);
+      if (*ptr == ',')
+	{
+	  ptr++;
+	  get_operand (&ptr, operand + 1, 1, DST);
+	}
+      else
+	{
+	  operand[1].mode = 0;
+	}
 
+      break;
     case 2:
       ptr++;
-      get_operand (&ptr, operand + 0, 0);
+      get_operand (&ptr, operand + 0, 0, SRC);
       if (*ptr == ',')
 	ptr++;
-      get_operand (&ptr, operand + 1, 1);
+      get_operand (&ptr, operand + 1, 1, DST);
       break;
 
     default:
@@ -550,99 +604,105 @@ DEFUN (get_operands, (noperands, op_end, operand),
    */
 static
 struct h8_opcode *
-DEFUN (get_specific, (opcode, operands),
-       struct h8_opcode *opcode AND
-       struct h8_op *operands)
-
+get_specific (opcode, operands)
+     struct h8_opcode *opcode;
+     struct h8_op *operands;
 {
   struct h8_opcode *this_try = opcode;
   int found = 0;
-  unsigned int noperands = opcode->noperands;
 
-  unsigned int dispreg;
   unsigned int this_index = opcode->idx;
 
   while (this_index == opcode->idx && !found)
     {
       unsigned int i;
+      found = 1;
 
       this_try = opcode++;
-      for (i = 0; i < noperands; i++)
+      for (i = 0; i < this_try->noperands && found; i++)
 	{
-	  op_type op = (this_try->args.nib[i]) & ~(B30 | B31);
+	  op_type op = this_try->args.nib[i];
+	  int x = operands[i].mode;
 
-	  switch (op)
+	  if ((op & (DISP | REG)) == (DISP | REG)
+	      && ((x & DISP | REG) == (DISP | REG)))
 	    {
-	    case Hex0:
-	    case Hex1:
-	    case Hex2:
-	    case Hex3:
-	    case Hex4:
-	    case Hex5:
-	    case Hex6:
-	    case Hex7:
-	    case Hex8:
-	    case Hex9:
-	    case HexA:
-	    case HexB:
-	    case HexC:
-	    case HexD:
-	    case HexE:
-	    case HexF:
-	      break;
-	    case DISPSRC:
-	    case DISPDST:
-	      operands[0].dispreg = operands[i].reg;
-	    case RD8:
-	    case RS8:
-	    case RDIND:
-	    case RSIND:
-	    case RD16:
-	    case RS16:
-	    case CCR:
-	    case RSINC:
-	    case RDDEC:
-	      if (operands[i].mode != op)
-		goto fail;
-	      break;
-	    case KBIT:
-	    case IMM16:
-	    case IMM3:
-	    case IMM8:
-	      if (operands[i].mode != IMM16)
-		goto fail;
-	      break;
-	    case MEMIND:
-	      if (operands[i].mode != MEMIND)
-		goto fail;
-	      break;
-	    case ABS16SRC:
-	    case ABS8SRC:
-	    case ABS16OR8SRC:
-	    case ABS16ORREL8SRC:
-	      if (operands[i].mode == DISP8)
+	      dispreg = operands[i].reg;
+	    }
+	  else if (op & REG)
+	    {
+	      if (!(x & REG))
+		found = 0;
+
+	      if (x & L_P)
 		{
-		  /* For compatibility with other assemblers, treat jsr 0xfff
-		     the same as jsr @0xfff */
+		  x = (x & ~L_P) | (Hmode ? L_32 : L_16);
+		}
+	      if (op & L_P)
+		{
+		  op = (op & ~L_P) | (Hmode ? L_32 : L_16);
+		}
+
+	      opsize = op & SIZE;
+
+	      /* The size of the reg is v important */
+	      if ((op & SIZE) != (x & SIZE))
+		found = 0;
+	    }
+	  else if ((op & ABSJMP) && (x & ABS))
+	    {
+	      operands[i].mode &= ~ABS;
+	      operands[i].mode |= ABSJMP;
+	      /* But it may not be 24 bits long */
+	      if (!Hmode)
+		{
+		  operands[i].mode &= ~SIZE;
+		  operands[i].mode |= L_16;
+		}
 
 
+	    }
+	  else if ((op & (KBIT | DBIT)) && (x & IMM))
+	    {
+	      /* This is ok if the immediate value is sensible */
+
+	    }
+	  else if (op & PCREL)
+	    {
+
+	      /* The size of the displacement is important */
+	      if ((op & SIZE) != (x & SIZE))
+		found = 0;
+
+	    }
+	  else if ((op & (DISP | IMM | ABS))
+		   && (op & (DISP | IMM | ABS)) == (x & (DISP | IMM | ABS)))
+	    {
+	      /* Got a diplacement,will fit if no size or same size as try */
+	      if (op & ABS && op & L_8) 
+		{
+		  /* We want an 8 bit abs here, but one which looks like 16 bits will do fine */
+		  if (x & L_16)
+		    found= 1;
 		}
 	      else
-		{
-		  if (operands[i].mode != ABS16SRC)
-		    goto fail;
-		}
-	      break;
-	    case ABS16OR8DST:
-	    case ABS16DST:
-	    case ABS8DST:
-	      if (operands[i].mode != ABS16DST)
-		goto fail;
-	      break;
+	      if ((x & SIZE) != 0
+		  && ((op & SIZE) != (x & SIZE)))
+		found = 0;
 	    }
+#if 0
+	  else if ((op & ABSMOV) && (x & ABS))
+	    {
+	      /* An absmov is only
+	      /* Ok */
+	    }
+#endif
+	  else if ((op & MODE) != (x & MODE))
+	    {
+	      found = 0;
+	    }
+
 	}
-      found = 1;
-    fail:;
     }
   if (found)
     return this_try;
@@ -651,23 +711,121 @@ DEFUN (get_specific, (opcode, operands),
 }
 
 static void
-DEFUN (check_operand, (operand, width, string),
-       struct h8_op *operand AND
-       unsigned int width AND
-       char *string)
+check_operand (operand, width, string)
+     struct h8_op *operand;
+     unsigned int width;
+     char *string;
 {
   if (operand->exp.X_add_symbol == 0
-      && operand->exp.X_subtract_symbol == 0)
+      && operand->exp.X_op_symbol == 0)
     {
 
       /* No symbol involved, let's look at offset, it's dangerous if any of
-         the high bits are not 0 or ff's, find out by oring or anding with
-         the width and seeing if the answer is 0 or all fs*/
+	 the high bits are not 0 or ff's, find out by oring or anding with
+	 the width and seeing if the answer is 0 or all fs*/
+      
       if ((operand->exp.X_add_number & ~width) != 0 &&
 	  (operand->exp.X_add_number | width) != (~0))
 	{
+	  if (width == 255 
+	      && (operand->exp.X_add_number & 0xff00) == 0xff00)
+	    {
+	      /* Just ignore this one - which happens when trying to 
+	      fit a 16 bit address truncated into an 8 bit address of something like bset  */
+	    }
+	  else 
+	    {
 	  as_warn ("operand %s0x%x out of range.", string, operand->exp.X_add_number);
 	}
+	}
+    }
+
+}
+
+static void
+do_a_fix_imm (offset, operand, relaxing)
+     int offset;
+     struct h8_op *operand;
+     int relaxing;
+{
+  int idx;
+  int size;
+  int where;
+
+
+  char *t = operand->mode & IMM ? "#" : "@";
+
+  if (operand->exp.X_add_symbol == 0)
+    {
+      char *bytes = frag_now->fr_literal + offset;
+      switch (operand->mode & SIZE)
+	{
+	case L_3:
+	  check_operand (operand, 0x7, t);
+	  bytes[0] |= (operand->exp.X_add_number) << 4;
+	  break;
+	case L_8:
+	  check_operand (operand, 0xff, t);
+	  bytes[0] = operand->exp.X_add_number;
+	  break;
+	case L_16:
+	  check_operand (operand, 0xffff, t);
+	  bytes[0] = operand->exp.X_add_number >> 8;
+	  bytes[1] = operand->exp.X_add_number >> 0;
+	  break;
+	case L_24:
+	  check_operand (operand, 0xffffff, t);
+	  bytes[0] = operand->exp.X_add_number >> 16;
+	  bytes[1] = operand->exp.X_add_number >> 8;
+	  bytes[2] = operand->exp.X_add_number >> 0;
+	  break;
+
+	case L_32:
+	  /* This should be done with bfd */
+	  bytes[0] = operand->exp.X_add_number >> 24;
+	  bytes[1] = operand->exp.X_add_number >> 16;
+	  bytes[2] = operand->exp.X_add_number >> 8;
+	  bytes[3] = operand->exp.X_add_number >> 0;
+	  break;
+	}
+
+    }
+  else
+    {
+      switch (operand->mode & SIZE)
+	{
+	default:
+	  abort ();
+	case L_24:
+	  size = 4;
+	  where = -1;
+	  idx = relaxing ? R_MOVLB1 : R_RELLONG;
+	  break;
+
+	case L_32:
+	  size = 4;
+	  where = 0;
+	  idx = R_RELLONG;
+	  break;
+	case L_16:
+	  size = 2;
+	  where = 0;
+	  idx = relaxing ? R_MOVB1 : R_RELWORD;
+	  break;
+	case L_8:
+	  size = 1;
+	  where = 0;
+	  idx = R_RELBYTE;
+	}
+
+
+      operand->exp.X_add_number = (short) operand->exp.X_add_number;
+      fix_new_exp (frag_now,
+		   offset + where,
+		   size,
+		   &operand->exp,
+		   0,
+		   idx);
     }
 
 }
@@ -675,10 +833,9 @@ DEFUN (check_operand, (operand, width, string),
 /* Now we know what sort of opcodes it is, lets build the bytes -
  */
 static void
-DEFUN (build_bytes, (this_try, operand),
-       struct h8_opcode *this_try AND
-       struct h8_op *operand)
-
+build_bytes (this_try, operand)
+     struct h8_opcode *this_try;
+     struct h8_op *operand;
 {
   unsigned int i;
 
@@ -688,21 +845,80 @@ DEFUN (build_bytes, (this_try, operand),
   char part;
   op_type c;
   char high;
+  unsigned int nibble_count = 0;
+  int absat;
+  int immat;
   int nib;
+  char asnibbles[30];
+  char *p = asnibbles;
+
+  if (!(this_try->inbase || Hmode))
+    {
+      as_warn ("Opcode `%s' only available on H8/300-H", this_try->name);
+    }
 
-top:;
   while (*nibble_ptr != E)
     {
-      int nibble;
+      int d;
+      c = *nibble_ptr++;
 
-      for (nibble = 0; nibble < 2; nibble++)
+      d = (c & DST) != 0;
+
+      if (c < 16)
 	{
-	  c = *nibble_ptr & ~(B30 | B31);
-	  switch (c)
+	  nib = c;
+	}
+      else
+	{
+
+	  if (c & (REG | IND | INC | DEC))
+	    {
+	      nib = operand[d].reg;
+	    }
+	  else if ((c & DISPREG) == (DISPREG))
+	    {
+	      nib = dispreg;
+	    }
+
+	  else if (c & ABSMOV)
+	    {
+	      operand[d].mode &= ~ABS;
+	      operand[d].mode |= ABSMOV;
+	      immat = nibble_count / 2;
+	      nib = 0;
+	    }
+	  else if (c &  ABS )
+	    {
+	      operand[d].mode = c;
+	      absat = nibble_count / 2;
+	      nib = 0;
+	    }
+	  else if (c & (IMM | PCREL | ABS | ABSJMP | DISP))
+	    {
+	      operand[d].mode = c;
+	      immat = nibble_count / 2;
+	      nib = 0;
+	    }
+	  else if (c & IGNORE)
+	    {
+	      nib = 0;
+	    }
+	  else if (c & DBIT)
+	    {
+	      switch (operand[0].exp.X_add_number)
+		{
+		case 1:
+		  nib = c;
+		  break;
+		case 2:
+		  nib = 0x8 | c;
+		  break;
+		default:
+		  as_bad ("Need #1 or #2 here");
+		}
+	    }
+	  else if (c & KBIT)
 	    {
-	    default:
-	      abort ();
-	    case KBIT:
 	      switch (operand[0].exp.X_add_number)
 		{
 		case 1:
@@ -711,148 +927,56 @@ top:;
 		case 2:
 		  nib = 8;
 		  break;
+		case 4:
+		  if (!Hmode)
+		    as_warn ("#4 only valid in h8/300 mode.");
+		  nib = 9;
+		  break;
+
 		default:
 		  as_bad ("Need #1 or #2 here");
 		  break;
 		}
 	      /* stop it making a fix */
 	      operand[0].mode = 0;
-	      break;
-	    case 0:
-	    case 1:
-	    case 2:
-	    case 3:
-	    case 4:
-	    case 5:
-	    case 6:
-	    case 7:
-	    case 8:
-	    case 9:
-	    case 10:
-	    case 11:
-	    case 12:
-	    case 13:
-	    case 14:
-	    case 15:
-	      nib = c;
-	      break;
-	    case DISPREG:
-	      nib = operand[0].dispreg;
-	      break;
-	    case IMM8:
-	      operand[0].mode = IMM8;
-	      nib = 0;
-	      break;
-
-	    case DISPDST:
-	      nib = 0;
-	      break;
-	    case IMM3:
-	      if (operand[0].exp.X_add_symbol == 0)
-		{
-		  operand[0].mode = 0;	/* stop it making a fix */
-		  nib = (operand[0].exp.X_add_number);
-		}
-	      else
-		as_bad ("can't have symbol for bit number");
-	      if (nib < 0 || nib > 7)
-		{
-		  as_bad ("Bit number out of range %d", nib);
-		}
-
-	      break;
-
-	    case ABS16DST:
-	      nib = 0;
-	      break;
-	    case ABS8DST:
-	      operand[1].mode = ABS8DST;
-	      nib = 0;
-	      break;
-	    case ABS8SRC:
-	      operand[0].mode = ABS8SRC;
-	      nib = 0;
-	      break;
-	    case ABS16OR8DST:
-	      operand[1].mode = c;
-
-	      nib = 0;
-
-	      break;
-
-	    case ABS16ORREL8SRC:
-	      operand[0].mode = c;
-	      nib = 0;
-	      break;
-
-	    case ABS16OR8SRC:
-	      operand[0].mode = ABS16OR8SRC;
-	      nib = 0;
-	      break;
-	    case DISPSRC:
-	      operand[0].mode = ABS16SRC;
-	      nib = 0;
-	      break;
-
-	    case DISP8:
-	      operand[0].mode = DISP8;
-	      nib = 0;
-	      break;
-
-	    case ABS16SRC:
-	    case IMM16:
-	    case IGNORE:
-	    case MEMIND:
-
-	      nib = 0;
-	      break;
-	    case RS8:
-	    case RS16:
-	    case RSIND:
-	    case RSINC:
-	      nib = operand[0].reg;
-	      break;
-
-	    case RD8:
-	    case RD16:
-	    case RDDEC:
-	    case RDIND:
-	      nib = operand[1].reg;
-	      break;
-
-	    case E:
-	      abort ();
-	      break;
 	    }
-	  if (*nibble_ptr & B31)
+
+	  if (c & B31)
 	    {
 	      nib |= 0x8;
 	    }
-
-	  if (nibble == 0)
-	    {
-	      *output_ptr = nib << 4;
-	    }
-	  else
-	    {
-	      *output_ptr |= nib;
-	      output_ptr++;
-	    }
-	  nibble_ptr++;
 	}
+      nibble_count++;
 
+      *p++ = nib;
+    }
+
+  for (i = 0; i < this_try->length; i++)
+    {
+      output[i] = (asnibbles[i * 2] << 4) | asnibbles[i * 2 + 1];
     }
 
   /* output any fixes */
   for (i = 0; i < 2; i++)
     {
-      switch (operand[i].mode)
-	{
-	case 0:
-	  break;
+      int x = operand[i].mode;
 
-	case DISP8:
-	  check_operand (operand + i, 0x7f, "@");
+      if (x & (IMM | DISP))
+	{
+	  do_a_fix_imm (output - frag_now->fr_literal + immat, operand + i, 0);
+	}
+      else if (x & ABS)
+	{
+	  do_a_fix_imm (output - frag_now->fr_literal + absat, operand + i, 0);
+	}
+      else if (x & PCREL)
+	{
+	  int size16 = x & L_16;
+	  int where = size16 ? 2 : 1;
+	  int size = size16 ? 2 : 1;
+	  int type = size16 ? R_PCRWORD : R_PCRBYTE;
+
+	  check_operand (operand + i, size16 ? 0x7fff : 0x7f, "@");
 
 	  if (operand[i].exp.X_add_number & 1)
 	    {
@@ -860,137 +984,50 @@ top:;
 		       operand->exp.X_add_number);
 	    }
 
-	  fix_new (frag_now,
-		   output - frag_now->fr_literal + 1,
-		   1,
-		   operand[i].exp.X_add_symbol,
-		   operand[i].exp.X_subtract_symbol,
-		   (char) (operand[i].exp.X_add_number - 1),
-		   1,
-		   R_PCRBYTE);
-	  break;
-	case IMM8:
-	  check_operand (operand + i, 0xff, "#");
-	  /* If there is nothing else going on we can safely
-             reloc in place */
-	  if (operand[i].exp.X_add_symbol == 0)
-	    {
-	      output[1] = operand[i].exp.X_add_number;
-	    }
-	  else
-	    {
-	      fix_new (frag_now,
+	  operand[i].exp.X_add_number =
+	    (char) (operand[i].exp.X_add_number - 1);
+	  fix_new_exp (frag_now,
+		       output - frag_now->fr_literal + where,
+		       size,
+		       &operand[i].exp,
+		       1,
+		       type);
+	}
+      else if (x & MEMIND)
+	{
+
+	  check_operand (operand + i, 0xff, "@@");
+	  fix_new_exp (frag_now,
 		       output - frag_now->fr_literal + 1,
 		       1,
-		       operand[i].exp.X_add_symbol,
-		       operand[i].exp.X_subtract_symbol,
-		       (short) (operand[i].exp.X_add_number),
+		       &operand[i].exp,
 		       0,
 		       R_RELBYTE);
-	    }
+	}
 
-	  break;
-	case MEMIND:
-	  check_operand (operand + i, 0xff, "@@");
-	  fix_new (frag_now,
-		   output - frag_now->fr_literal + 1,
-		   1,
-		   operand[i].exp.X_add_symbol,
-		   operand[i].exp.X_subtract_symbol,
-		   operand[i].exp.X_add_number,
-		   0,
-		   R_RELBYTE);
-	  break;
-	case ABS8DST:
-	case ABS8SRC:
-#if 0
-	  /* Unwanted*/
-	  check_operand (operand + i, 0xff, "@");
-#endif
-	  fix_new (frag_now,
-		   output - frag_now->fr_literal + 1,
-		   1,
-		   operand[i].exp.X_add_symbol,
-		   operand[i].exp.X_subtract_symbol,
-		   (char) (operand[i].exp.X_add_number),
-		   0,
-		   R_RELBYTE);
-	  break;
+      else if (x & ABSMOV)
+	{
+	  /* This mov is either absolute long or thru a memory loc */
+	  do_a_fix_imm (output - frag_now->fr_literal + immat, operand + i, 1);
+	}
 
-	case ABS16OR8SRC:
-	case ABS16OR8DST:
+      else if (x & ABSJMP)
+	{
+	  /* This jmp may be a jump or a branch */
 
-	  check_operand (operand + i, 0xffff, "@");
-
-	  fix_new (frag_now,
-		   output - frag_now->fr_literal + 2,
-		   2,
-		   operand[i].exp.X_add_symbol,
-		   operand[i].exp.X_subtract_symbol,
-		   (short) (operand[i].exp.X_add_number),
-		   0,
-		   R_MOVB1);
-	  break;
-
-	case ABS16ORREL8SRC:
-	  check_operand (operand + i, 0xffff, "@");
+	  check_operand (operand + i, Hmode ? 0xfffff : 0xffff, "@");
 	  if (operand[i].exp.X_add_number & 1)
 	    {
 	      as_warn ("branch operand has odd offset (%x)\n",
 		       operand->exp.X_add_number);
 	    }
-	  fix_new (frag_now,
-		   output - frag_now->fr_literal + 2,
-		   2,
-		   operand[i].exp.X_add_symbol,
-		   operand[i].exp.X_subtract_symbol,
-		   (short) (operand[i].exp.X_add_number),
-		   0,
-		   R_JMP1);
-	  break;
-
-
-	case ABS16SRC:
-	case ABS16DST:
-	case IMM16:
-	case DISPSRC:
-	case DISPDST:
-	  check_operand (operand + i, 0xffff, "@");
-	  if (operand[i].exp.X_add_symbol == 0)
-	    {
-	      /* This should be done with bfd */
-	      output[3] = operand[i].exp.X_add_number & 0xff;
-	      output[2] = operand[i].exp.X_add_number >> 8;
-
-	    }
-	  else
-	    {
-
-	      fix_new (frag_now,
-		       output - frag_now->fr_literal + 2,
-		       2,
-		       operand[i].exp.X_add_symbol,
-		       operand[i].exp.X_subtract_symbol,
-		       (short) (operand[i].exp.X_add_number),
+	  operand[i].exp.X_add_number = (short) operand[i].exp.X_add_number;
+	  fix_new_exp (frag_now,
+		       output - frag_now->fr_literal,
+		       4,
+		       &operand[i].exp,
 		       0,
-		       R_RELWORD);
-	    }
-
-	  break;
-	case RS8:
-	case RD8:
-	case RS16:
-	case RD16:
-	case RDDEC:
-	case KBIT:
-	case RSINC:
-	case RDIND:
-	case RSIND:
-	case CCR:
-
-	  break;
-	default:
-	  abort ();
+		       R_JMPL1);
 	}
     }
 
@@ -1243,7 +1280,7 @@ md_atof (type, litP, sizeP)
       md_number_to_chars (litP, (long) (*wordP++), sizeof (LITTLENUM_TYPE));
       litP += sizeof (LITTLENUM_TYPE);
     }
-  return "";			/* Someone should teach Dean about null pointers */
+  return 0;
 }
 
 int
@@ -1269,8 +1306,8 @@ tc_aout_fix_to_chars ()
 void
 md_create_short_jump (ptr, from_addr, to_addr, frag, to_symbol)
      char *ptr;
-     long from_addr;
-     long to_addr;
+     addressT from_addr;
+     addressT to_addr;
      fragS *frag;
      symbolS *to_symbol;
 {
@@ -1280,7 +1317,7 @@ md_create_short_jump (ptr, from_addr, to_addr, frag, to_symbol)
 void
 md_create_long_jump (ptr, from_addr, to_addr, frag, to_symbol)
      char *ptr;
-     long from_addr, to_addr;
+     addressT from_addr, to_addr;
      fragS *frag;
      symbolS *to_symbol;
 {
@@ -1297,10 +1334,10 @@ md_convert_frag (headers, fragP)
   abort ();
 }
 
-long
-DEFUN (md_section_align, (seg, size),
-       segT seg AND
-       long size)
+valueT 
+md_section_align (seg, size)
+     segT seg;
+     valueT size;
 {
   return ((size + (1 << section_alignment[(int) seg]) - 1) & (-1 << section_alignment[(int) seg]));
 
@@ -1354,7 +1391,7 @@ md_estimate_size_before_relax (fragP, segment_type)
 void
 DEFUN (md_number_to_chars, (ptr, use, nbytes),
        char *ptr AND
-       long use AND
+       valueT use AND
        int nbytes)
 {
   switch (nbytes)
@@ -1433,4 +1470,9 @@ tc_reloc_mangle (fix_ptr, intr, base)
 
 }
 
+tc_coff_sizemachdep ()
+{
+  abort ();
+}
+
 /* end of tc-h8300.c */
