@@ -25,11 +25,12 @@
 #define TC_COFF_FIX2RTYPE(fixP) abort();
 
 #define BFD_ARCH bfd_arch_h8300
-#define COFF_MAGIC 0x8300
+#define COFF_MAGIC Hmode ? 0x8301 : 0x8300
 #define TC_COUNT_RELOC(x) (1)
 #define IGNORE_NONSTANDARD_ESCAPES
 
 #define TC_RELOC_MANGLE(a,b,c) tc_reloc_mangle(a,b,c)
+#define TC_CONS_RELOC          (Hmode ? R_RELLONG: R_RELWORD)
 
 #define DO_NOT_STRIP 0
 #define DO_STRIP 0
@@ -38,4 +39,7 @@
 #define RELOC_32 1234
 #define COFF_FLAGS 1
 
+extern int Hmode;
+
 /* end of tc-h8300.h */
+
