@@ -1,5 +1,5 @@
 /* tc-i860.c -- Assemble for the I860
-   Copyright (C) 1989, 1992 Free Software Foundation, Inc.
+   Copyright (C) 1989, 1992, 1993 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -37,7 +37,7 @@ void md_ri_to_chars ();
 static void i860_ip ();
 /* void emit_machine_reloc(); */
 
-int md_reloc_size = sizeof (struct relocation_info);
+const int md_reloc_size = sizeof (struct relocation_info);
 
 /* void (*md_emit_relocations)() = emit_machine_reloc; */
 
@@ -190,7 +190,7 @@ md_begin ()
     {
       const char *name = i860_opcodes[i].name;
       retval = hash_insert (op_hash, name, &i860_opcodes[i]);
-      if (retval != NULL && *retval != '\0')
+      if (retval != NULL)
 	{
 	  fprintf (stderr, "internal error: can't hash `%s': %s\n",
 		   i860_opcodes[i].name, retval);
@@ -257,7 +257,7 @@ md_assemble (str)
 
 	case E_MOV:
 	  if (the_insn.exp.X_add_symbol == NULL &&
-	      the_insn.exp.X_subtract_symbol == NULL &&
+	      the_insn.exp.X_op_symbol == NULL &&
 	      (the_insn.exp.X_add_number < (1 << 15) &&
 	       the_insn.exp.X_add_number >= -(1 << 15)))
 	    break;
@@ -273,7 +273,7 @@ md_assemble (str)
 
 	case E_ADDR:
 	  if (the_insn.exp.X_add_symbol == NULL &&
-	      the_insn.exp.X_subtract_symbol == NULL)
+	      the_insn.exp.X_op_symbol == NULL)
 	    break;
 	  /* orh ha%addr_expr,r0,r31 */
 	  pseudo[0].opcode = 0xec000000 | (atmp << 16);
@@ -287,7 +287,7 @@ md_assemble (str)
 
 	case E_U32:		/* 2nd version emulates Intel as, not doc. */
 	  if (the_insn.exp.X_add_symbol == NULL &&
-	      the_insn.exp.X_subtract_symbol == NULL &&
+	      the_insn.exp.X_op_symbol == NULL &&
 	      (the_insn.exp.X_add_number < (1 << 16) &&
 	       the_insn.exp.X_add_number >= 0))
 	    break;
@@ -309,7 +309,7 @@ md_assemble (str)
 
 	case E_AND:		/* 2nd version emulates Intel as, not doc. */
 	  if (the_insn.exp.X_add_symbol == NULL &&
-	      the_insn.exp.X_subtract_symbol == NULL &&
+	      the_insn.exp.X_op_symbol == NULL &&
 	      (the_insn.exp.X_add_number < (1 << 16) &&
 	       the_insn.exp.X_add_number >= 0))
 	    break;
@@ -333,7 +333,7 @@ md_assemble (str)
 
 	case E_S32:
 	  if (the_insn.exp.X_add_symbol == NULL &&
-	      the_insn.exp.X_subtract_symbol == NULL &&
+	      the_insn.exp.X_op_symbol == NULL &&
 	      (the_insn.exp.X_add_number < (1 << 15) &&
 	       the_insn.exp.X_add_number >= -(1 << 15)))
 	    break;
@@ -377,9 +377,7 @@ md_assemble (str)
 	  fix_new (frag_now,	/* which frag */
 		   (toP - frag_now->fr_literal),	/* where */
 		   4,		/* size */
-		   the_insn.exp.X_add_symbol,
-		   the_insn.exp.X_subtract_symbol,
-		   the_insn.exp.X_add_number,
+		   &the_insn.exp,
 		   the_insn.pcrel,
 	  /* merge bit fields into one argument */
 	  (int) (((the_insn.highlow & 0x3) << 4) | (the_insn.reloc & 0xf)));
@@ -763,20 +761,11 @@ getExpression (str)
 
   save_in = input_line_pointer;
   input_line_pointer = str;
-  switch (seg = expression (&the_insn.exp))
+  seg = expression (&the_insn.exp);
+  if (seg != absolute_section
+      && seg != undefined_section
+      && ! SEG_NORMAL (seg))
     {
-
-    case SEG_ABSOLUTE:
-    case SEG_TEXT:
-    case SEG_DATA:
-    case SEG_BSS:
-    case SEG_UNKNOWN:
-    case SEG_DIFFERENCE:
-    case SEG_BIG:
-    case SEG_ABSENT:
-      break;
-
-    default:
       the_insn.error = "bad segment";
       expr_end = input_line_pointer;
       input_line_pointer = save_in;
@@ -852,7 +841,7 @@ md_atof (type, litP, sizeP)
       md_number_to_chars (litP, (long) (*wordP++), sizeof (LITTLENUM_TYPE));
       litP += sizeof (LITTLENUM_TYPE);
     }
-  return "";			/* Someone should teach Dean about null pointers */
+  return 0;
 }
 
 /*
@@ -861,7 +850,7 @@ md_atof (type, litP, sizeP)
 void
 md_number_to_chars (buf, val, n)
      char *buf;
-     long val;
+     valueT val;
      int n;
 {
   switch (n)
@@ -993,7 +982,7 @@ md_number_to_imm (buf, val, n, fixP)
 void
 md_create_short_jump (ptr, from_addr, to_addr, frag, to_symbol)
      char *ptr;
-     long from_addr, to_addr;
+     addressT from_addr, to_addr;
      fragS *frag;
      symbolS *to_symbol;
 {
@@ -1062,7 +1051,7 @@ md_convert_frag (headers, fragP)
 void
 md_create_long_jump (ptr, from_addr, to_addr, frag, to_symbol)
      char *ptr;
-     long from_addr, to_addr;
+     addressT from_addr, to_addr;
      fragS *frag;
      symbolS *to_symbol;
 {
@@ -1117,10 +1106,10 @@ print_insn (insn)
 	   insn->exp.X_add_symbol ?
 	   (S_GET_NAME (insn->exp.X_add_symbol) ?
 	    S_GET_NAME (insn->exp.X_add_symbol) : "???") : "0");
-  fprintf (stderr, "\t\tX_sub_symbol = %s\n",
-	   insn->exp.X_subtract_symbol ?
-	   (S_GET_NAME (insn->exp.X_subtract_symbol) ?
-	    S_GET_NAME (insn->exp.X_subtract_symbol) : "???") : "0");
+  fprintf (stderr, "\t\tX_op_symbol = %s\n",
+	   insn->exp.X_op_symbol ?
+	   (S_GET_NAME (insn->exp.X_op_symbol) ?
+	    S_GET_NAME (insn->exp.X_op_symbol) : "???") : "0");
   fprintf (stderr, "\t\tX_add_number = %d\n",
 	   insn->exp.X_add_number);
   fprintf (stderr, "}\n");
@@ -1287,10 +1276,10 @@ md_undefined_symbol (name)
 }
 
 /* Round up a section size to the appropriate boundary.  */
-long
+valueT
 md_section_align (segment, size)
      segT segment;
-     long size;
+     valueT size;
 {
   return size;			/* Byte alignment is fine */
 }
