@@ -67,12 +67,13 @@
 
 /*
  *  The code and its structures are re-enterent.
+ *
  *  Before you do anything else, you must call hash_new() which will
- *  return the address of a hash-table-control-block (or NULL if there
- *  is not enough memory). You then use this address as a handle of the
- *  symbol table by passing it to all the other hash_...() functions.
- *  The only approved way to recover the memory used by the symbol table
- *  is to call hash_die() with the handle of the symbol table.
+ *  return the address of a hash-table-control-block.  You then use
+ *  this address as a handle of the symbol table by passing it to all
+ *  the other hash_...() functions.  The only approved way to recover
+ *  the memory used by the symbol table is to call hash_die() with the
+ *  handle of the symbol table.
  *
  *  Before you call hash_die() you normally delete anything pointed to
  *  by individual symbols. After hash_die() you can't use that symbol
@@ -103,8 +104,7 @@
  *    (total hashes,collisions) for (reads,writes) (*)
  *  All of the above values vary in time.
  *  (*) some of these numbers will not be meaningful if we change the
- *  internals.
- */
+ *  internals. */
 
 /*
  *  I N T E R N A L
@@ -136,7 +136,7 @@
 
 #define error	as_fatal
 
-#define DELETED     ((char *)1)	/* guarenteed invalid address */
+#define DELETED     ((PTR)1)	/* guarenteed invalid address */
 #define START_POWER    (11)	/* power of two: size of new hash table */
 
 /* TRUE if a symbol is in entry @ ptr.  */
@@ -179,7 +179,7 @@ Panic! Please make #include "stat.h" agree with previous definitions!
   char                * s;        symbol string (address) [ key ]
   char                * v;        value string (address)  [datum]
   boolean               f;        TRUE if we found s in hash table            i
-  char                * t;        error string; "" means OK
+  char                * t;        error string; 0 means OK
   int                   a;        access type [0...n)                         i
 
   c=hash_new       ()             create new hash_control
@@ -234,55 +234,43 @@ Panic! Please make #include "stat.h" agree with previous definitions!
    hash_jam().  TRUE: we replaced a value FALSE: we inserted a value.  */
 static char hash_found;
 
-static struct hash_entry *hash_ask ();
-static int hash_code ();
-static char *hash_grow ();
+static struct hash_entry *hash_ask PARAMS ((struct hash_control *,
+					    const char *, int));
+static int hash_code PARAMS ((struct hash_control *, const char *));
+static const char *hash_grow PARAMS ((struct hash_control *));
 
 /* Create a new hash table.  Return NULL if failed; otherwise return handle
    (address of struct hash).  */
 struct hash_control *
 hash_new ()
 {
-  register struct hash_control *retval;
-  register struct hash_entry *room;	/* points to hash table */
-  register struct hash_entry *wall;
-  register struct hash_entry *entry;
-  register int *ip;		/* scan stats block of struct hash_control */
-  register int *nd;		/* limit of stats block */
+  struct hash_control *retval;
+  struct hash_entry *room;	/* points to hash table */
+  struct hash_entry *wall;
+  struct hash_entry *entry;
+  int *ip;		/* scan stats block of struct hash_control */
+  int *nd;		/* limit of stats block */
 
-  room = (struct hash_entry *) malloc (sizeof (struct hash_entry)
-				       /* +1 for the wall entry */
-				       * ((1 << START_POWER) + 1));
-  if (room != NULL)
-    {
-      retval = (struct hash_control *) malloc (sizeof (struct hash_control));
-      if (retval != NULL)
-	{
-	  nd = retval->hash_stat + STATLENGTH;
-	  for (ip = retval->hash_stat; ip < nd; ip++)
-	    {
-	      *ip = 0;
-	    }
+  room = (struct hash_entry *) xmalloc (sizeof (struct hash_entry)
+					/* +1 for the wall entry */
+					* ((1 << START_POWER) + 1));
+  retval = (struct hash_control *) xmalloc (sizeof (struct hash_control));
 
-	  retval->hash_stat[STAT_SIZE] = 1 << START_POWER;
-	  retval->hash_mask = (1 << START_POWER) - 1;
-	  retval->hash_sizelog = START_POWER;
-	  /* works for 1's compl ok */
-	  retval->hash_where = room;
-	  retval->hash_wall =
-	    wall = room + (1 << START_POWER);
-	  retval->hash_full = (1 << START_POWER) / 2;
-	  for (entry = room; entry <= wall; entry++)
-	    {
-	      entry->hash_string = NULL;
-	    }
-	}
-    }
-  else
-    {
-      retval = NULL;		/* no room for table: fake a failure */
-    }
-  return (retval);		/* return NULL or set-up structs */
+  nd = retval->hash_stat + STATLENGTH;
+  for (ip = retval->hash_stat; ip < nd; ip++)
+    *ip = 0;
+
+  retval->hash_stat[STAT_SIZE] = 1 << START_POWER;
+  retval->hash_mask = (1 << START_POWER) - 1;
+  retval->hash_sizelog = START_POWER;
+  /* works for 1's compl ok */
+  retval->hash_where = room;
+  retval->hash_wall =
+    wall = room + (1 << START_POWER);
+  retval->hash_full = (1 << START_POWER) / 2;
+  for (entry = room; entry <= wall; entry++)
+    entry->hash_string = NULL;
+  return retval;
 }
 
 /*
@@ -318,12 +306,12 @@ hash_die (handle)
  */
 void
 hash_say (handle, buffer, bufsiz)
-     register struct hash_control *handle;
-     register int buffer[ /*bufsiz*/ ];
-     register int bufsiz;
+     struct hash_control *handle;
+     int buffer[ /*bufsiz*/ ];
+     int bufsiz;
 {
-  register int *nd;		/* limit of statistics block */
-  register int *ip;		/* scan statistics */
+  int *nd;		/* limit of statistics block */
+  int *ip;		/* scan statistics */
 
   ip = handle->hash_stat;
   nd = ip + min (bufsiz - 1, STATLENGTH);
@@ -346,14 +334,14 @@ hash_say (handle, buffer, bufsiz)
  * Anyway, the symbol is not present after this function.
  *
  */
-char *				/* NULL if string not in table, else */
+PTR				/* NULL if string not in table, else */
 /* returns value of deleted symbol */
 hash_delete (handle, string)
-     register struct hash_control *handle;
-     register char *string;
+     struct hash_control *handle;
+     const char *string;
 {
-  register char *retval;
-  register struct hash_entry *entry;
+  PTR retval;
+  struct hash_entry *entry;
 
   entry = hash_ask (handle, string, STAT__WRITE);
   if (hash_found)
@@ -383,14 +371,14 @@ hash_delete (handle, string)
  * Return NULL and don't change the table if the symbol is not already
  * in the table.
  */
-char *
+PTR
 hash_replace (handle, string, value)
-     register struct hash_control *handle;
-     register char *string;
-     register char *value;
+     struct hash_control *handle;
+     const char *string;
+     PTR value;
 {
-  register struct hash_entry *entry;
-  register char *retval;
+  struct hash_entry *entry;
+  char *retval;
 
   entry = hash_ask (handle, string, STAT__WRITE);
   if (hash_found)
@@ -403,32 +391,32 @@ hash_replace (handle, string, value)
       retval = NULL;
     }
   ;
-  return (retval);
+  return retval;
 }
 
 /*
  *                   h a s h _ i n s e r t ( )
  *
  * Insert a (symbol-string, value) into the hash table.
- * Return an error string, "" means OK.
+ * Return an error string, 0 means OK.
  * It is an 'error' to insert an existing symbol.
  */
 
-char *				/* return error string */
+const char *			/* return error string */
 hash_insert (handle, string, value)
-     register struct hash_control *handle;
-     register char *string;
-     register char *value;
+     struct hash_control *handle;
+     const char *string;
+     PTR value;
 {
-  register struct hash_entry *entry;
-  register char *retval;
+  struct hash_entry *entry;
+  const char *retval;
 
-  retval = "";
+  retval = 0;
   if (handle->hash_stat[STAT_USED] > handle->hash_full)
     {
       retval = hash_grow (handle);
     }
-  if (!*retval)
+  if (!retval)
     {
       entry = hash_ask (handle, string, STAT__WRITE);
       if (hash_found)
@@ -442,7 +430,7 @@ hash_insert (handle, string, value)
 	  handle->hash_stat[STAT_USED] += 1;
 	}
     }
-  return (retval);
+  return retval;
 }
 
 /*
@@ -451,7 +439,7 @@ hash_insert (handle, string, value)
  * Regardless of what was in the symbol table before, after hash_jam()
  * the named symbol has the given value. The symbol is either inserted or
  * (its value is) relpaced.
- * An error message string is returned, "" means OK.
+ * An error message string is returned, 0 means OK.
  *
  * WARNING: this may decide to grow the hashed symbol table.
  * To do this, we call hash_grow(), WHICH WILL recursively CALL US.
@@ -459,21 +447,21 @@ hash_insert (handle, string, value)
  * We report status internally: hash_found is TRUE if we replaced, but
  * false if we inserted.
  */
-char *
+const char *
 hash_jam (handle, string, value)
-     register struct hash_control *handle;
-     register char *string;
-     register char *value;
+     struct hash_control *handle;
+     const char *string;
+     PTR value;
 {
-  register char *retval;
-  register struct hash_entry *entry;
+  const char *retval;
+  struct hash_entry *entry;
 
-  retval = "";
+  retval = 0;
   if (handle->hash_stat[STAT_USED] > handle->hash_full)
     {
       retval = hash_grow (handle);
     }
-  if (!*retval)
+  if (!retval)
     {
       entry = hash_ask (handle, string, STAT__WRITE);
       if (!hash_found)
@@ -483,7 +471,7 @@ hash_jam (handle, string, value)
 	}
       entry->hash_value = value;
     }
-  return (retval);
+  return retval;
 }
 
 /*
@@ -491,52 +479,54 @@ hash_jam (handle, string, value)
  *
  * Grow a new (bigger) hash table from the old one.
  * We choose to double the hash table's size.
- * Return a human-scrutible error string: "" if OK.
+ * Return a human-scrutible error string: 0 if OK.
  * Warning! This uses hash_jam(), which had better not recurse
  * back here! Hash_jam() conditionally calls us, but we ALWAYS
  * call hash_jam()!
  * Internal.
  */
-static char *
+static const char *
 hash_grow (handle)		/* make a hash table grow */
      struct hash_control *handle;
 {
-  register struct hash_entry *newwall;
-  register struct hash_entry *newwhere;
+  struct hash_entry *newwall;
+  struct hash_entry *newwhere;
   struct hash_entry *newtrack;
-  register struct hash_entry *oldtrack;
-  register struct hash_entry *oldwhere;
-  register struct hash_entry *oldwall;
-  register int temp;
+  struct hash_entry *oldtrack;
+  struct hash_entry *oldwhere;
+  struct hash_entry *oldwall;
+  int temp;
   int newsize;
-  char *string;
-  char *retval;
+  const char *string;
+  const char *retval;
 #ifdef SUSPECT
   int oldused;
 #endif
 
   /*
-	 * capture info about old hash table
-	 */
+   * capture info about old hash table
+   */
   oldwhere = handle->hash_where;
   oldwall = handle->hash_wall;
 #ifdef SUSPECT
   oldused = handle->hash_stat[STAT_USED];
 #endif
   /*
-	 * attempt to get enough room for a hash table twice as big
-	 */
+   * attempt to get enough room for a hash table twice as big
+   */
   temp = handle->hash_stat[STAT_SIZE];
-  if ((newwhere = (struct hash_entry *)
-  xmalloc ((long) ((temp + temp + 1) * sizeof (struct hash_entry)))) != NULL)
+  if ((newwhere = ((struct hash_entry *)
+		   xmalloc ((unsigned long) ((temp + temp + 1)
+					     * sizeof (struct hash_entry)))))
+      != NULL)
     /* +1 for wall slot */
     {
-      retval = "";		/* assume success until proven otherwise */
+      retval = 0;		/* assume success until proven otherwise */
       /*
-		     * have enough room: now we do all the work.
-		     * double the size of everything in handle,
-		     * note: hash_mask frob works for 1's & for 2's complement machines
-		     */
+       * have enough room: now we do all the work.
+       * double the size of everything in handle,
+       * note: hash_mask frob works for 1's & for 2's complement machines
+       */
       handle->hash_mask = handle->hash_mask + handle->hash_mask + 1;
       handle->hash_stat[STAT_SIZE] <<= 1;
       newsize = handle->hash_stat[STAT_SIZE];
@@ -547,50 +537,45 @@ hash_grow (handle)		/* make a hash table grow */
       handle->hash_wall =
 	newwall = newwhere + newsize;
       /*
-		     * set all those pesky new slots to vacant.
-		     */
+       * set all those pesky new slots to vacant.
+       */
       for (newtrack = newwhere; newtrack <= newwall; newtrack++)
 	{
 	  newtrack->hash_string = NULL;
 	}
       /*
-		     * we will do a scan of the old table, the hard way, using the
-		     * new control block to re-insert the data into new hash table.
-		     */
+       * we will do a scan of the old table, the hard way, using the
+       * new control block to re-insert the data into new hash table.
+       */
       handle->hash_stat[STAT_USED] = 0;	/* inserts will bump it up to correct */
       for (oldtrack = oldwhere; oldtrack < oldwall; oldtrack++)
-	{
-	  if (((string = oldtrack->hash_string) != NULL) && string != DELETED)
-	    {
-	      if (*(retval = hash_jam (handle, string, oldtrack->hash_value)))
-		{
-		  break;
-		}
-	    }
-	}
+	if (((string = oldtrack->hash_string) != NULL) && string != DELETED)
+	  if ((retval = hash_jam (handle, string, oldtrack->hash_value)))
+	    break;
+
 #ifdef SUSPECT
-      if (!*retval && handle->hash_stat[STAT_USED] != oldused)
+      if (!retval && handle->hash_stat[STAT_USED] != oldused)
 	{
 	  retval = "hash_used";
 	}
 #endif
-      if (!*retval)
+      if (!retval)
 	{
 	  /*
-				 * we have a completely faked up control block.
-				 * return the old hash table.
-				 */
+	   * we have a completely faked up control block.
+	   * return the old hash table.
+	   */
 	  free ((char *) oldwhere);
 	  /*
-				 * Here with success. retval is already "".
-				 */
+	   * Here with success. retval is already 0.
+	   */
 	}
     }
   else
     {
       retval = "no room";
     }
-  return (retval);
+  return retval;
 }
 
 /*
@@ -644,8 +629,8 @@ hash_apply (handle, function)
      struct hash_control *handle;
      char *(*function) ();
 {
-  register struct hash_entry *entry;
-  register struct hash_entry *wall;
+  struct hash_entry *entry;
+  struct hash_entry *wall;
 
   wall = handle->hash_wall;
   for (entry = handle->hash_where; entry < wall; entry++)
@@ -664,24 +649,18 @@ hash_apply (handle, function)
  * Given symbol string, find value (if any).
  * Return found value or NULL.
  */
-char *
-hash_find (handle, string)	/* return char* or NULL */
+PTR
+hash_find (handle, string)
      struct hash_control *handle;
-     char *string;
+     const char *string;
 {
-  register struct hash_entry *entry;
-  register char *retval;
+  struct hash_entry *entry;
 
   entry = hash_ask (handle, string, STAT__READ);
   if (hash_found)
-    {
-      retval = entry->hash_value;
-    }
+    return entry->hash_value;
   else
-    {
-      retval = NULL;
-    }
-  return (retval);
+    return NULL;
 }
 
 /*
@@ -696,14 +675,14 @@ hash_find (handle, string)	/* return char* or NULL */
 static struct hash_entry *	/* string slot, may be empty or deleted */
 hash_ask (handle, string, access)
      struct hash_control *handle;
-     char *string;
+     const char *string;
      int access;		/* access type */
 {
-  register char *string1;	/* JF avoid strcmp calls */
-  register char *s;
-  register int c;
-  register struct hash_entry *slot;
-  register int collision;	/* count collisions */
+  const char *string1;	/* JF avoid strcmp calls */
+  const char *s;
+  int c;
+  struct hash_entry *slot;
+  int collision;	/* count collisions */
 
   /* start looking here */
   slot = handle->hash_where + hash_code (handle, string);
@@ -776,11 +755,11 @@ hash_ask (handle, string, access)
 static int
 hash_code (handle, string)
      struct hash_control *handle;
-     register char *string;
+     const char *string;
 {
-  register long h;		/* hash code built here */
-  register long c;		/* each character lands here */
-  register int n;		/* Amount to shift h by */
+  long h;		/* hash code built here */
+  long c;		/* each character lands here */
+  int n;		/* Amount to shift h by */
 
   n = (handle->hash_sizelog - 3);
   h = 0;
@@ -818,11 +797,8 @@ int number;			/* number 0:TABLES-1 of current hashed */
 main ()
 {
   char (*applicatee ());
-  char *hash_find ();
   char *destroy ();
   char *what ();
-  struct hash_control *hash_new ();
-  char *hash_replace ();
   int *ip;
 
   number = 0;
@@ -876,14 +852,15 @@ main ()
 	  break;
 	case 'i':
 	  p = hash_insert (h, name = what ("symbol"), value = what ("value"));
-	  if (*p)
+	  if (p)
 	    {
-	      printf ("symbol=\"%s\"  value=\"%s\"  error=%s\n", name, value, p);
+	      printf ("symbol=\"%s\"  value=\"%s\"  error=%s\n", name, value,
+		      p);
 	    }
 	  break;
 	case 'j':
 	  p = hash_jam (h, name = what ("symbol"), value = what ("value"));
-	  if (*p)
+	  if (p)
 	    {
 	      printf ("symbol=\"%s\"  value=\"%s\"  error=%s\n", name, value, p);
 	    }
